@@ -835,7 +835,8 @@ Widget::constructed() noexcept
         gtk_event_controller_set_name(controller.get(), "vte-motion-controller");
         gtk_widget_add_controller(m_widget, controller.release());
 
-        auto const scroll_flags = GtkEventControllerScrollFlags(GTK_EVENT_CONTROLLER_SCROLL_VERTICAL);
+        auto const scroll_flags = GtkEventControllerScrollFlags(GTK_EVENT_CONTROLLER_SCROLL_VERTICAL |
+                                                                GTK_EVENT_CONTROLLER_SCROLL_HORIZONTAL);
         controller = vte::glib::take_ref(gtk_event_controller_scroll_new(scroll_flags));
         g_signal_connect(controller.get(), "scroll-begin",
                          G_CALLBACK(scroll_begin_cb), this);
@@ -1729,6 +1730,23 @@ Widget::notify_scroll_value_changed()
 #endif // VTE_GTK == 4
 }
 
+void
+Widget::notify_termprops_changed(int const* props,
+                                 int n_props) noexcept
+{
+        m_in_termprops_changed_emission = true;
+
+        auto retval = gboolean{};
+        g_signal_emit(object(),
+                      signals[SIGNAL_TERMPROPS_CHANGED],
+                      0, /* detail */
+                      props,
+                      n_props,
+                      &retval);
+
+        m_in_termprops_changed_emission = false;
+}
+
 #if VTE_GTK == 3
 
 std::optional<ScrollEvent>
